@import url('https://fonts.googleapis.com/css2?family=Squada+One&display=swap');
@import url('https://fonts.googleapis.com/css2?family=Oswald&display=swap');
@import url('https://fonts.googleapis.com/css2?family=Cairo&display=swap');

body { background: #d6ca96 }
body, .tooltip { font-family: 'Cairo', sans-serif; }
h1 { font-family: 'Squada One', sans-serif; margin:0; padding-top: 10px; }
h2, h3 { font-family: 'Oswald', sans-serif; margin:0;}

nav ul {
 list-style-type: none;
 margin: 0; padding: 0;
 display: flex; }
nav ul li { padding: 4px; margin-right: 1px; }
nav ul li:before { content: " | "; }
nav ul li:first-child:before { content: ""; }
nav ul li.active { background-color: #cfcfcf; }

a { color: #333; }
a:hover { color: #0162c2ff; }

#progress { float:left; border-right: solid 1px #333;
 margin-right:10px; padding-right: 10px;
 height:90%;}

#whoami { position:absolute; top: 20px; right: calc(100% - 480px); }

#ws { position:absolute; top: 30px; left: 490px;
 width: 10px; height: 10px; border-radius: 10px;
 border: solid #333 1px; background: #999; }
.has-tooltip:hover .tooltip { visibility: visible;
   font-weight:normal;
   font-size:10pt; }
.tooltip { visibility: hidden; background: white;
 border: solid #333 1px; padding: 4px;  position:absolute }

.diff pre { display: block;  margin: 0; padding: 0; }
.diff del { color: #000; background: #fdd; }
.diff ins { color: #000; background: #dfd; }
