<div id="test-result">

  <h1>All known tests have passed</h1>

  <p>This may be a good time to commit your changes,
     or spend some time improving your code.</p>


  <p>When you're ready, run <code>tanco next</code>
     to start work on the next feature.</p>

</div>
