<h2>result</h2>

<h2>test: {{ test.name }} {{ test.head }}</h2>

<pre>{{  test.body }}</pre>

<h3>input lines</h3>
<pre>{% for line in test.ilines %}{{ line }}
{% endfor %}</pre>

{% if result.kind.name == 'Pass' %}
  <h3>pass</h3>
{% else %}
<h3>diff</h3>
<div class="diff">
{% for line in result.error.to_data()['data']['diff'] %}
  {% if line.startswith('-') %}
    <pre><del>{{ line }}</del></pre>
  {% elif line.startswith('+') %}
    <pre><ins>{{ line }}</ins></pre>
  {% else %}
    <pre>{{ line }}</pre>
  {% endif %}
{% endfor %}
</div>
{% endif %}


<h3>actual</h3>
<pre>{% for line in result.actual %}{{ line }}
{% endfor %}</pre>

<pre>
{{ result.to_data() | pprint }}
</pre>
