
<title>{{ data.name }}</title>

<h2>{{ data.name }} : {{ data.head }}</h2>

<pre>
{{ data.body }}
</pre>

<h3>input</h3>

<pre>
{{ data.ilines }}
</pre>


<h3>expected output</h3>
<pre>
{{ data.olines }}
</pre>
