<title>user: {{ data.username }}</title>

<h2>user: {{ data.username }}</h2>


<h3>attempts</h3>
<ul hx-target="#main">
{% for a in data.attempts %}
  <li>
    <a class="has-tooltip" href="/a/{{ a.code }}"
    >{{ a.c_name }}<span class="tooltip">{{ a.code }}</span></a>
    started {{ a.ts }}
  </li>
{% endfor %}
</ul>
